import string

import maya.cmds as cmds

from vtool.maya_lib import api as api_old
from vtool.maya_lib import attr
from vtool.maya_lib import core
from vtool import util



def zero_out(transform):
    if cmds.objExists('%s.offsetParentMatrix' % transform):
        if cmds.nodeType(transform) == 'transform':
            matrix = cmds.getAttr('%s.matrix' % transform)
            cmds.setAttr('%s.offsetParentMatrix' % transform, matrix, type = 'matrix')
            
    cmds.xform(transform, t = [0,0,0], ro = [0,0,0], s = [1,1,1], shear = [0,0,0])
    
    if cmds.nodeType(transform) == 'joint':
        cmds.setAttr('%s.jointOrientX' % transform, 0)        
        cmds.setAttr('%s.jointOrientY' % transform, 0)
        cmds.setAttr('%s.jointOrientZ' % transform, 0)
    
def attach(transform_source, transform_target):
    
    mult_matrix = cmds.createNode('multMatrix', n = 'multMatrix_%s' % transform_target)
        
    cmds.connectAttr('%s.worldMatrix' % transform_source, '%s.matrixIn[0]' % mult_matrix)
    
    parent = cmds.listRelatives(transform_target, p = True)
    if parent:
        cmds.connectAttr('%s.worldInverseMatrix' % parent[0], '%s.matrixIn[1]' % mult_matrix)
    
    zero_out(transform_target)
    
    
    input_attr = attr.get_attribute_input('%s.offsetParentMatrix' % transform_target)
    
    if not input_attr:
        cmds.connectAttr('%s.matrixSum' % mult_matrix, '%s.offsetParentMatrix' % transform_target)
    
    blend_matrix = None
    
    if input_attr:
        input_node_type = cmds.nodeType(input_attr)
        
        if not input_node_type == 'blendMatrix':
            blend_matrix = cmds.createNode('blendMatrix')
            cmds.connectAttr('%s.outputMatrix' % blend_matrix, '%s.offsetParentMatrix' % transform_target, f = True)
            cmds.connectAttr(input_attr, '%s.target[0].targetMatrix' % blend_matrix)
        
        if input_node_type == 'blendMatrix':
            blend_matrix = core.get_basename(input_attr, remove_namespace = False, remove_attribute = True)
            
        
        next_slot = attr.get_available_slot('%s.target' % blend_matrix )
        cmds.connectAttr('%s.matrixSum' % mult_matrix, '%s.target[%s].targetMatrix' % (blend_matrix, next_slot))
    
    
    return [mult_matrix, blend_matrix]
       
def blend_matrix_switch(blend_matrix_node, attribute_name = 'switch',attribute_names = [], attribute_node = None):
    
    
    
    blend_matrix_node = util.convert_to_sequence(blend_matrix_node)
    
    if not blend_matrix_node[0] or not cmds.objExists(blend_matrix_node[0]):
        return
    
    condition_dict = {}
    
    if not attribute_node:
        attribute_node = blend_matrix_node[0]
    
    indices = attr.get_indices('%s.target' % blend_matrix_node[0])
    
    names = []
    for index in indices:
        name = 'switch%s' % (index + 1)
        if index < len(attribute_names)-1:
            name = attribute_names[index]
        names.append(name)
    
    enum_name = string.join(names, ':')
    
    cmds.addAttr(attribute_node, ln = attribute_name, k = True, at = 'enum', en = enum_name)
    source_attribute = '%s.%s' % (attribute_node, attribute_name)
    
    for node in blend_matrix_node:
        if not node or not cmds.objExists(node):
            continue
    
        indices = attr.get_indices('%s.target' % node)
        
        inc = 0
        
        for index in indices:
            
            if condition_dict.has_key(index):
                condition = condition_dict[index]
            else:
                condition = cmds.createNode('condition')
                cmds.connectAttr(source_attribute, '%s.firstTerm' % condition)
                cmds.setAttr('%s.secondTerm' % condition, inc)
            
                cmds.setAttr('%s.colorIfTrueR' % condition, 1)
                cmds.setAttr('%s.colorIfFalseR' % condition, 0)
                condition_dict[inc] = condition
            
            cmds.connectAttr('%s.outColorR' % condition, '%s.target[%s].weight' % (node, index))
                    
            inc += 1
    